
#ifndef _VECTEUR3X_H_
#define _VECTEUR3X_H_ 1

#include <iostream>	

using namespace std;

#define DEGTORAD(degree) ((degree) * (3.141592654f / 180.0f))
#define RADTODEG(radian) ((radian) * (180.0f / 3.141592654f))

template <class T>
class vecteur3X{
public :
	T x;
	T y;
	T z;
	vecteur3X();
	vecteur3X(T*);
	vecteur3X(T,T,T);
	void set(T,T,T);
	void set(T*);
	T getX() const;
	T getY() const;
	T getZ() const;
	void setX(T);
	void setY(T);
	void setZ(T);
	void actX(T);
	void actY(T);
	void actZ(T);
	void getPos(T*) const;
	T* getPos() const;
	T* getPos4() const;
	void normaliz();

    vecteur3X<T> operator * (const T scalar);
    friend vecteur3X<T> operator * (const T scalar, const vecteur3X<T> &other);

    vecteur3X<T>& operator = (const vecteur3X<T> &other);
    vecteur3X<T>& operator += (const vecteur3X<T> &other);
    vecteur3X<T>& operator -= (const vecteur3X<T> &other);
	
    static float distance(const vecteur3X<T> &v1, const vecteur3X<T> &v2);
    static T dotProduct(const vecteur3X<T> &v1,  const vecteur3X<T> &v2 );
    static vecteur3X<T> crossProduct(const vecteur3X<T> &v1, const vecteur3X<T> &v2);
};

//***********************************************************************************

template <class T>
inline vecteur3X<T>::vecteur3X(){
	x=(T)0;
	y=(T)0;
	z=(T)0;
};

template <class T>
inline vecteur3X<T>::vecteur3X(T x1,T y1,T z1){
	x=x1;
	y=y1;
	z=z1;
};

template <class T>
inline vecteur3X<T>::vecteur3X(T* vals){
	set(vals);
};

template <class T>
inline void vecteur3X<T>::set(T x1,T y1,T z1){
	x=x1;
	y=y1;
	z=z1;
};

template <class T>
inline void vecteur3X<T>::set(T *val){
	x=val[0];y=val[1];z=val[2];
};

template <class T>
inline T *vecteur3X<T>::getPos4() const{
	T *sortie=new T[4];
	sortie[0]=x;sortie[1]=y;sortie[2]=z;sortie[3]=(T)1;
	return sortie;
};

template <class T>
inline T *vecteur3X<T>::getPos() const{
	T *sortie=new T[3];
	sortie[0]=x;sortie[1]=y;sortie[2]=z;
	return sortie;
};

template <class T>
inline void vecteur3X<T>::getPos(T *sortie) const{
	sortie[0]=x;sortie[1]=y;sortie[2]=z;
};

template <class T>
inline T vecteur3X<T>::getX() const{
	return x;
};

template <class T>
inline T vecteur3X<T>::getY() const{
	return y;
};

template <class T>
inline T vecteur3X<T>::getZ() const{
	return z;
};

template <class T>
inline void vecteur3X<T>::setX(T x1){
	x=x1;
};

template <class T>
inline void vecteur3X<T>::setY(T y1){
	y=y1;
};

template <class T>
inline void vecteur3X<T>::setZ(T z1){
	z=z1;
};

template <class T>
inline void vecteur3X<T>::actX(T x1){
	x+=x1;
};

template <class T>
inline void vecteur3X<T>::actY(T y1){
	y+=y1;
};

template <class T>
inline void vecteur3X<T>::actZ(T z1){
	z+=z1;
};

template <class T>
inline void vecteur3X<T>::normaliz(){
	T fLength = sqrt( x * x + y * y + z * z );
    x =(T) x / fLength;
    y =(T) y / fLength;
    z =(T) z / fLength;
};

template <class T>
inline vecteur3X<T> vecteur3X<T>::operator * ( const T scalar )
{
	return vecteur3X(x * scalar,y * scalar,z * scalar);
};

template <class T>
inline vecteur3X<T> operator * ( const T scalar, const vecteur3X<T> &other )
{
	return vecteur3X(other.x * scalar,other.y * scalar,other.z * scalar);
};

template <class T>
inline vecteur3X<T>& vecteur3X<T>::operator = ( const vecteur3X<T> &other )
{
    x = other.x;
    y = other.y;
    z = other.z;

    return *this;
}

template <class T>
inline vecteur3X<T>& vecteur3X<T>::operator += ( const vecteur3X<T> &other )
{
    x += other.x;
    y += other.y;
    z += other.z;

    return *this;
}

template <class T>
inline vecteur3X<T>& vecteur3X<T>::operator -= ( const vecteur3X<T> &other )
{
    x -= other.x;
    y -= other.y;
    z -= other.z;

    return *this;
}


template <class T>
inline float distance( const vecteur3X<T> &v1,  const vecteur3X<T> &v2  )
{
    float dx = v1.x - v2.x;
    float dy = v1.y - v2.y;
    float dz = v1.z - v2.z;

    return (float)sqrt( dx * dx + dy * dy + dz * dz );
}

template <class T>
inline T dotProduct( const vecteur3X<T> &v1,  const vecteur3X<T> &v2 )
{
    return( v1.x * v2.x + v1.y * v2.y + v1.z * v2.z  );
}

template <class T>
inline vecteur3X<T> crossProduct( const vecteur3X<T> &v1,  const vecteur3X<T> &v2 )
{
    return vecteur3X<T>(v1.y * v2.z - v1.z * v2.y,
		v1.z * v2.x - v1.x * v2.z,
		v1.x * v2.y - v1.y * v2.x);
}

#endif // _VECTEUR3X_H_
